/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, US$

    Copyright (C) 2008-2008 bla <blapost@gmail.com>
*/
//---------------------------------------------------------------------------
#pragma hdrstop
#include <stdlib.h>
#include "crapto1.h"
#include "crypto1.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
#define SWAPENDIAN(x)\
	(x = (x >> 8 & 0xff00ff) | (x & 0xff00ff) << 8, x = x >> 16 | x << 16)

struct Crypto1State * crypto1_create(unsigned __int64 key)
{
	struct Crypto1State *s = (struct Crypto1State*) malloc(sizeof(*s));
	int i;

	for(i = s->odd = s->even = 0; i < 48; i += 2) {
		s->odd  = s->odd  << 1 | BIT(key, i + 1);
		s->even = s->even << 1 | BIT(key, i);
	}
	return s;
}
void crypto1_destroy(struct Crypto1State *state)
{
	free(state);
}
void crypto1_get_lfsr(struct Crypto1State *state, unsigned __int64 *lfsr)
{
	int i;

	for(*lfsr = i = 0; i < 24; ++i)	{
		*lfsr = *lfsr << 1 | BIT(state->odd, i);
		*lfsr = *lfsr << 1 | BIT(state->even, i);
	}
}
unsigned int crypto1_word(struct Crypto1State *s, unsigned int in_word, int fb)
{
	unsigned int i, ret = 0, feedback;

	for (i = 0; i < 32; ++i) {
		ret |= (feedback = filter(s->odd)) << i;

		feedback &= !!fb;
		feedback ^= BEBIT(in_word, i);
		feedback ^= LF_POLY_ODD & s->odd;
		feedback ^= LF_POLY_EVEN & s->even;
		s->even = s->even << 1 | parity(feedback);

		s->odd ^= (s->odd ^= s->even, s->even ^= s->odd);
	}

	return SWAPENDIAN(ret);
}

/* prng_successor
 * helper used to obscure the keystream during authentication
 */
unsigned int prng_successor(unsigned int x, unsigned int n)
{
	SWAPENDIAN(x);
	while(n--)
		x = x >> 1 | (x >> 16 ^ x >> 18 ^ x >> 19 ^ x >> 21) << 31;

	return SWAPENDIAN(x);
}

